% =================================================
% =================================================
% Christmas Tree - static version
% author: Hanchu Wang

% This code is for plotting 2 chirstmas trees based on Conical Spiral
% Function.
% =================================================
% =================================================

close all; clear all; clc
% =================================================
% Color RGB
% =================================================
bl = [ 22,  42, 114]/255;
dg = [ 22,  91,  51]/255;
gr = [ 20, 107,  58]/255;
oy = [248, 178,  41]/255;
or = [234,  70,  48]/255;
rd = [187,  37,  40]/255;
sg = [ 86, 125,  48]/255;
mg = [143, 177,  78]/255;
dy = [249, 243,  85]/255;

% =================================================
% Conical Spiral - Parameters and function
% Equation reference: Weisstein, Eric W. "Conical Spiral." From MathWorld--A Wolfram Web Resource. https://mathworld.wolfram.com/ConicalSpiral.html
% =================================================

h = 10;
r = h/1.618/2;
a = 100;

fx = @(h,r,a,z) (h-z)./h.*r.*cos(a.*z);
fy = @(h,r,a,z) (h-z)./h.*r.*sin(a.*z);

% =================================================
% Conical Spiral - Parameters and function
% =================================================
mainbody_z = 0:0.01:10;
mainbody_x = fx(h,r,a,mainbody_z);
mainbody_y = fy(h,r,a,mainbody_z);

ball_z=0.1:0.2:10;
ball_x=fx(h,r,a,ball_z);
ball_y=fy(h,r,a,ball_z);

leaf_z1=0.1:0.01:5;
leaf_x1=fx(h,r,a*2,leaf_z1)+0.1;
leaf_y1=fy(h,r,a*2,leaf_z1)+0.1;

stringLight_z=0.1:0.01:10;
stringLight_x=fx(h,r+0.1,a*4,stringLight_z+0.01);
stringLight_y=fy(h,r+0.1,a*4,stringLight_z+0.01);

stringLight2_z=0:0.12:10;
stringLight2_x=fx(h,r,a,stringLight2_z);
stringLight2_y=fy(h,r,a,stringLight2_z);


leaf_z2=0.1:0.015:10;
leaf_x2=fx(h,r,a,leaf_z2+0.01);
leaf_y2=fy(h,r,a,leaf_z2+0.01);

snow=rand(50,3);
snow(:,1:2)=snow(:,1:2).*((4-(-4))+1)+(-4.5);
snow(:,3)=snow(:,3).*(10+1);

snow2=rand(105,3);
snow2(:,1:2)=snow2(:,1:2).*((4-(-4))+1)+(-4.5);
snow2(:,3)=snow2(:,3).*(10+1);

% =================================================
% Plotting
% =================================================
figure (1)

% tree
plot3(mainbody_x,mainbody_y,mainbody_z, '*', 'Color',gr,'LineWidth', 1.5)

hold on
% star
plot3(0,0,h+0.2,'p', 'MarkerSize',28,'MarkerFaceColor',oy, 'MarkerEdgeColor',oy.*0.8, 'LineWidth', 2)
scatter3(0,0,h+0.2,20000, 'w', 'MarkerFaceColor', oy, 'MarkerEdgeAlpha', 0, 'MarkerFaceAlpha', 0.1);

%leaf
plot3(leaf_x1,leaf_y1,leaf_z1,'*','Color', gr);
plot3(leaf_x2,leaf_y2,leaf_z2,'*','Color', sg);

%string lights
plot3(stringLight_x,stringLight_y,stringLight_z,'.', 'Color', dy);
scatter3(stringLight_x,stringLight_y,stringLight_z,900, '.', 'MarkerFaceColor', dy, 'MarkerEdgeColor', dy, 'MarkerEdgeAlpha', 0, 'MarkerFaceAlpha', 0.1);

%balls
plot3(ball_x,ball_y,ball_z, 'o','MarkerSize',10, 'Color',or, 'MarkerFaceColor', or, 'MarkerEdgeColor',rd)

% snow
plot3(snow(:,1),snow(:,2),snow(:,3),'*','Color',[1 1 1])
plot3(snow2(:,1),snow2(:,2),snow2(:,3),'.','Color',[1 1 1], 'MarkerSize',5)

% text
text(2,2,8,{['Merry Christmas']},'HorizontalAlignment','left','FontSize',20,'Color', [1 1 1],'fontname','Britannic Bold');

% =================================================
% Formatting
% =================================================
pbaspect([1 1 1])
xlim([-4.5 4.5])
ylim([-4.5 4.5])
zlim([0 11])

set(gca,'Color',bl*0.2)
set(gcf,'Color','w')
set(gcf, 'Position',  [100, 100, 1000, 1000])

hold off

figure (2)

% tree
plot3(mainbody_x,mainbody_y,mainbody_z, '-', 'Color',gr,'LineWidth', 1.5)

hold on
% star
plot3(0,0,h+0.2,'p', 'MarkerSize',28,'MarkerFaceColor',oy, 'MarkerEdgeColor',oy.*0.8, 'LineWidth', 2)
scatter3(0,0,h+0.2,20000, 'w', 'MarkerFaceColor', oy, 'MarkerEdgeAlpha', 0, 'MarkerFaceAlpha', 0.1);

%leaf
plot3(leaf_x1,leaf_y1,leaf_z1,'-','Color', gr);
plot3(leaf_x2,leaf_y2,leaf_z2,'-','Color', sg);

%string lights
plot3(stringLight2_x,stringLight2_y,stringLight2_z,':', 'Color', dy,  'LineWidth', 2);

%balls
plot3(ball_x,ball_y,ball_z, 'o','MarkerSize',10, 'Color',or, 'MarkerFaceColor', or, 'MarkerEdgeColor',rd)

% snow
plot3(snow(:,1),snow(:,2),snow(:,3),'*','Color',[1 1 1])
plot3(snow2(:,1),snow2(:,2),snow2(:,3),'.','Color',[1 1 1], 'MarkerSize',5)

% text
text(2,2,8,{['Merry Christmas']},'HorizontalAlignment','left','FontSize',20,'Color', [1 1 1],'fontname','Britannic Bold');

% =================================================
% Formatting
% =================================================
pbaspect([1 1 1])
xlim([-4.5 4.5])
ylim([-4.5 4.5])
zlim([0 11])

% -------------------------------------------------
% polygon frame
set(gca,'Color',bl*0.2)
set(gcf,'Color','w')
% if wants no frame, use following lines instead
% set(gcf,'Color',bl*0.2)
% -------------------------------------------------

%optional
set(gcf, 'Position',  [1000, 100, 1000, 1000])

hold off
% End--------------------------------------------------------------------
